﻿-- 
-- ACP - Addon Control Panel (Formerly Master Control Program)
--
-- Allows you to control AddOn loading after logging in.
-- 
-- Personal Use Version by Sylvaan - Proudmoore US/Alliance
--
--

local _G = getfenv(0)

ACP_SelectedProfile = "NONE"

UIPanelWindows["ACP_AddonList"] = { area = "center", pushable = 0, whileDead = 1 }

StaticPopupDialogs["ACP_RELOADUI"] = {
	text = ACP_RELOAD,
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	OnAccept = function()
		ReloadUI()
	end,
	timeout = 0,
	hideOnEscape = 1
}

function ACP_DeleteDialog()
	if not StaticPopupDialogs["ACP_DELETEPROFILE"] then
		StaticPopupDialogs["ACP_DELETEPROFILE"] = {
			text = ACP_DELETE_PROFILE_DIALOG..ACP_SelectedProfile,
			button1 = TEXT(ACCEPT),
			button2 = TEXT(CANCEL),
			OnAccept = function()
				ACP_DeleteProfile(ACP_SelectedProfile)
				ACP_SelectedProfile = "NONE"
				UIDropDownMenu_SetText(ACP_SelectedProfile, ACP_AddonList_ProfileSelection)
			end,
			timeout = 0,
			hideOnEscape = 1
		}
	end
	StaticPopup_Show("ACP_DELETEPROFILE")
end

function ACP_SaveDialog()
	if not StaticPopupDialogs["ACP_SAVEPROFILE"] then
		StaticPopupDialogs["ACP_SAVEPROFILE"] = {
			text = ACP_PROFILE_NAME_SAVE,
			button1 = TEXT(ACCEPT),
			button2 = TEXT(CANCEL),
			hasEditBox = 1,
			maxLetters = 32,
			hideOnEscape = 1,
			OnLoad = function()
				_G[this:GetParent():GetName().."EditBox"]:SetText(ACP_SelectedProfile)
			end,
			OnAccept = function()
				ACP_SaveProfile(_G[this:GetParent():GetName().."EditBox"]:GetText())
			end,
			timeout = 0,
			EditBoxOnEnterPressed = function()
				ACP_SaveProfile(_G[this:GetParent():GetName().."EditBox"]:GetText())
				this:GetParent():Hide()
			end
		}
	end
	StaticPopup_Show("ACP_SAVEPROFILE")
end

ACP_VERSION = "2007.01.31"
ACP_LINEHEIGHT = 16
local ACP_MAXADDONS = 20
local ACP_BLIZZARD_ADDONS = { 
	"Blizzard_AuctionUI",
	"Blizzard_BattlefieldMinimap",
	"Blizzard_BindingUI",
	"Blizzard_CombatText",
	"Blizzard_CraftUI",
	"Blizzard_GMSurveyUI",
	"Blizzard_InspectUI",
	"Blizzard_ItemSocketingUI",
	"Blizzard_MacroUI",
	"Blizzard_RaidUI",
	"Blizzard_TalentUI",
	"Blizzard_TradeSkillUI",
	"Blizzard_TrainerUI",
}
local ACP_BLIZZARD_ADDONS_TITLES = { 
	"Blizzard: Auction",
	"Blizzard: Battlefield Minimap",
	"Blizzard: Binding",
	"Blizzard: Combat Text",
	"Blizzard: Craft",
	"Blizzard: GM Survey",
	"Blizzard: Inspect",
	"Blizzard: Item Socketing",
	"Blizzard: Macro",
	"Blizzard: Raid",
	"Blizzard: Talent",
	"Blizzard: Trade Skill",
	"Blizzard: Trainer",
}


function ACP_OnLoad()
	-- setup /ACP slash command
	SlashCmdList["ACPSLASHCMD"] = ACP_SlashHandler
	SLASH_ACPSLASHCMD1 = "/ACP"
end

-- /ACP slash command, only shows the UI
function ACP_SlashHandler(msg)
	ShowUIPanel(ACP_AddonList)
end

function ACP_AddonList_Enable(index,enabled)
	if type(index) == "number" then
		if enabled then
			ACP_EnableRecurse(index)
		else
			DisableAddOn(index)
		end
	else
		if enabled then
			if type(ACP_Config) == "table" and type(ACP_Config.refusetoload) == "table" then
				ACP_Config.refusetoload[index] = nil
			end
		else
			if type(ACP_Config) ~= "table" then ACP_Config = {} end
			if type(ACP_Config.refusetoload) ~= "table" then ACP_Config.refusetoload = {} end
			ACP_Config.refusetoload[index] = true
		end
	end
	ACP_AddonList_OnShow()
end

function ACP_AddonList_LoadNow(index)
	UIParentLoadAddOn(index)
	ACP_AddonList_OnShow()
end

function ACP_AddonList_OnShow()
	local function setSecurity(obj, idx)
		local width,height,iconWidth = 64,16,16
		local increment = iconWidth/width
		local left = (idx-1)*increment
		local right = idx*increment
		obj:SetTexCoord(left, right, 0, 1)
	end

	local numAddons = GetNumAddOns()
	local origNumAddons = numAddons
	numAddons = numAddons + #(ACP_BLIZZARD_ADDONS)
	FauxScrollFrame_Update(ACP_AddonList_ScrollFrame, numAddons, ACP_MAXADDONS, ACP_LINEHEIGHT, nil, nil, nil)
	local i
	local offset = FauxScrollFrame_GetOffset(ACP_AddonList_ScrollFrame)
	for i = 1, ACP_MAXADDONS, 1 do
		obj = _G["ACP_AddonListEntry"..i]
		local addonIdx = offset+i
		if addonIdx > numAddons then
			obj:Hide()
			obj.addon = nil
		else
			obj:Show()
			local titleText = _G["ACP_AddonListEntry"..i.."Title"]
			local status = _G["ACP_AddonListEntry"..i.."Status"]
			local checkbox = _G["ACP_AddonListEntry"..i.."Enabled"]
			local securityIcon = _G["ACP_AddonListEntry"..i.."SecurityIcon"]
			local loadnow = _G["ACP_AddonListEntry"..i.."LoadNow"]

			local name, title, notes, enabled, loadable, reason, security
			if addonIdx > origNumAddons then
				name = ACP_BLIZZARD_ADDONS[(addonIdx-origNumAddons)]
				obj.addon = name
				title = ACP_BLIZZARD_ADDONS_TITLES[(addonIdx-origNumAddons)]
				notes = ""
				if type(ACP_Config) == "table" and type(ACP_Config.refusetoload) == "table" and ACP_Config.refusetoload[name] then
					enabled = nil
					loadable = nil
					reason = "WILL_NOT_LOAD"
				else
					enabled = 1
					loadable = 1
				end
				if (IsAddOnLoaded(name)) then
					reason = "LOADED"
					loadable = 1
				end
				security = "SECURE"
			else
				name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(addonIdx)
				obj.addon = addonIdx
			end
			local loaded = IsAddOnLoaded(name)
			local ondemand = IsAddOnLoadOnDemand(name)
			if loadable then
				titleText:SetTextColor(1,0.78,0)
			elseif enabled and reason ~= "DEP_DISABLED" then
				titleText:SetTextColor(1,0.1,0.1)
			else
				titleText:SetTextColor(0.5,0.5,0.5)
			end
			if title then
				titleText:SetText(title)
			else
				titleText:SetText(name)
			end
			if title == "Addon Control Panel"  then
				checkbox:Hide()
			else
				checkbox:Show()
				checkbox:SetChecked(enabled)
			end
			if security == "SECURE" then
				setSecurity(securityIcon,1)
			elseif security == "INSECURE" then
				setSecurity(securityIcon,2)
			elseif security == "BANNED" then
				setSecurity(securityIcon,3)
			end
			if reason then
				status:SetText(TEXT(_G["ACP_ADDON_"..reason]))
			elseif loaded then
				status:SetText(TEXT(ACP_ADDON_LOADED))
			elseif ondemand then
				status:SetText(ACP_LOADED_ON_DEMAND)
			else
				status:SetText("")
			end
			if not loaded and enabled and ondemand then
				loadnow:Show()
			else
				loadnow:Hide()
			end
		end
	end
end

function ACP_SaveProfile(profile)
	if profile == "NONE" then return end

	local numAddons = GetNumAddOns()
	local i

	-- make sure we have a config for profile setup
	if type(ACP_Config) ~= "table" then ACP_Config = {} end
	if type(ACP_Config.profiles) ~= "table" then ACP_Config.profiles = {} end

	-- setup this profile, and clear any previous data in case it exists
	ACP_Config.profiles[profile] = {}

	-- set each addon to be loaded or not loaded
	for i = 1, numAddons do
		local name, _, _, enabled = GetAddOnInfo(i)
		if enabled then
			ACP_Config.profiles[profile][name] = 1
		else
			ACP_Config.profiles[profile][name] = 0
		end
	end

	if not StaticPopupDialogs["ACP_PROFILESAVED"] then
		StaticPopupDialogs["ACP_PROFILESAVED"] = {
			text=ACP_PROFILE_SAVED..profile,
			button1 = TEXT(OK),
			OnAccept = function() end,
			timeout = 2,
			hideOnEscape = 1
		}
	end

	StaticPopup_Show("ACP_PROFILESAVED")
end

function ACP_LoadProfile()
	local i
	local addons

	ACP_SelectedProfile = this.value
	if not ACP_SelectedProfile or ACP_SelectedProfile == "" then
		ACP_SelectedProfile = "NONE"
		return
	end


	for addons in pairs(ACP_Config.profiles[ACP_SelectedProfile]) do
		if ACP_Config.profiles[ACP_SelectedProfile][addons] == 1 then
			EnableAddOn(addons)
		else
			DisableAddOn(addons)
		end
	end


	if type(ACP_Config) ~= "table" then ACP_Config = {} end
	ACP_Config.SelectedProfile = ACP_SelectedProfile
	UIDropDownMenu_SetText(ACP_SelectedProfile, ACP_AddonList_ProfileSelection)

	ACP_AddonList_OnShow()
end


function ACP_DeleteProfile(profile)
	if type(ACP_Config) ~= "table" then ACP_Config = {} end
	if type(ACP_Config.profiles) ~= "table" then ACP_Config.profiles = {} end
	local buttontext = ACP_PROFILE_DELETED..profile

	if profile == "NONE" then
		buttontext = ACP_NO_PROFILE_DELETED
	else
		ACP_Config.profiles[profile] = nil
	end

	ACP_DeleteDialog()
end


function ACP_ResetProfiles(param)
	UIDropDownMenu_Initialize(this,  ACP_InitializeProfiles)
	UIDropDownMenu_SetWidth(220, this)

	if type(ACP_Config) == "table" and type(ACP_Config.SelectedProfile) == "table" then
		ACP_SelectedProfile = ACP_Config.SelectedProfile
		UIDropDownMenu_SetText(ACP_SelectedProfile, ACP_AddonList_ProfileSelection)
	end
end

function ACP_InitializeProfiles()
	if type(ACP_Config) == "table" and type(ACP_Config.profiles) == "table" then
		local info = {}
		local profile

		for profile in pairs(ACP_Config.profiles) do
			info = {
				["text"] = profile,
				["value"] = profile,
				["func"] = ACP_LoadProfile,
			}
			UIDropDownMenu_AddButton(info)
		end
	end
end

function ACP_EnableAll()
	local numAddons = GetNumAddOns()
	local i

	-- set each addon to be enabled
	for i = 1, numAddons do
		local name, _, _, enabled = GetAddOnInfo(i)
		if not enabled then
			EnableAddOn(name)
		end
	end
	ACP_AddonList_OnShow()
end

function ACP_DisableAll()
	local numAddons = GetNumAddOns()
	local i

	-- set each addon to be enabled
	for i = 1, numAddons do
		local name, title, _, enabled = GetAddOnInfo(i)
		if enabled and title ~= "Addon Control Panel" then
			DisableAddOn(name)
		end
	end
	ACP_AddonList_OnShow()
end

function ACP_TooltipShow(index)
	local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(index)

	local version = GetAddOnMetadata(name, "Version")

	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMLEFT")

	if version then
		if version:find("%$Revision: (%d+) %$") then
			version = version:gsub("%$Revision: (%d+) %$", "%1")
		elseif version:find("%$Rev: (%d+) %$") then
			version = version:gsub("%$Rev: (%d+) %$", "%1")
		elseif version:find("%$LastChangedRevision: (%d+) %$") then
			version = version:gsub("%$LastChangedRevision: (%d+) %$", "%1")
		end
		version = version:trim()
	end

	local x
	if title then
		x = tostring(title)
	elseif name then
		x = "|cffffff7f" .. tostring(name) .. "|r"
	else
		x = "|cffffff7f<" .. tostring(class) .. " instance>|r"
	end
	if version then
		x = x .. " - |cffffff7f" .. tostring(version) .. "|r"
	end
	GameTooltip:AddLine(x, nil, nil, nil, nil, true)

	local author, credits, ldate
	author = GetAddOnMetadata(name, "Author")
	credits = GetAddOnMetadata(name, "X-Credits")
	ldate = GetAddOnMetadata(name, "X-Date") or GetAddOnMetadata(name, "X-ReleaseDate")

	if ldate then
		if ldate:find("%$Date: (.-) %$") then
			ldate = ldate:gsub("%$Date: (.-) %$", "%1")
		elseif ldate:find("%$LastChangedDate: (.-) %$") then
			ldate = ldate:gsub("%$LastChangedDate: (.-) %$", "%1")
		end
	end

	if notes then
		GameTooltip:AddLine("|cffffff7f" .. tostring(notes):trim() .. "|r", nil, nil, nil, true)
	end
	if author then
		GameTooltip:AddLine(" - |cffffff7f" .. "Author" .. ":|r " .. tostring(author):trim(), nil, nil, nil, true)
	end
	if credits then
		GameTooltip:AddLine(" - |cffffff7f" .. "Credits" .. ":|r " .. tostring(credits):trim(), nil, nil, nil, true)
	end
	if ldate then
		GameTooltip:AddLine(" - |cffffff7f" .. "Date" .. ":|r " .. tostring(ldate):trim(), nil, nil, nil, true)
	end


	local category = GetAddOnMetadata(name, "X-Category")
	if category then
		GameTooltip:AddLine(" - |cffffff7f" .. "Category" .. ":|r " .. category:trim(), nil, nil, nil, true)
	end

	local RequiredDeps = build_string(GetAddOnDependencies(index))
	if RequiredDeps then
		GameTooltip:AddLine(" - |cffffff7f" .. "Dependancies" .. ":|r " .. RequiredDeps:trim(), nil, nil, nil, true)
	end

	local metaXEmbeds = GetAddOnMetadata(name, "X-Embeds")
	if metaXEmbeds then
		GameTooltip:AddLine(" - |cffffff7f" .. "Embeds" .. ":|r " .. metaXEmbeds:trim(), nil, nil, nil, true)
	end

	if reason then
		GameTooltip:AddLine(" - |cffff7f7f" .. "Reason" .. ":|r " .. tostring(_G["ADDON_"..reason]):trim(), nil, nil, nil, true)
	end

	GameTooltip:Show()
end

function build_string(...)
	local s
	for i=1,select("#", ...) do
		local x = select(i, ...)
		if x and x:len() then
			s = s and (s..", "..x) or x
		end
	end
	return s
end

function ACP_EnableRecurse(name)
	EnableAddOn(name)
	enable_lod_dependants(name)

	recursive_iterate_over(GetAddOnDependencies(name))

	local metaXEmbeds = GetAddOnMetadata(name, "X-Embeds")
	if metaXEmbeds then
		recursive_iterate_over(strsplit(" ,", metaXEmbeds:trim()))
	end
end

addon_status = {}
addon_status_idx = {}
function ACP_UpdateAddonTable()
	for i=1, GetNumAddons() do
		if type(addon_status[i]) ~= "table" then
			addon_status[i] = {}
		end
		local s = addon_status[i]
		s.name, s.title, s.notes, s.enabled, s.loadable, s.reason, s.security = GetAddOnInfo(i)
	end
	for i,v in ipairs(addon_status) do
		addon_status_idx[v.name] = i
	end
end

function iterate_over(...)
	for i=1,select("#", ...) do
		local x = select(i, ...)
		if x and x:len() then
			EnableAddOn(x)
		end
	end
end

function find_iterate_over(name, ...)
	for i=1,select("#", ...) do
		local x = select(i, ...)
		if x and x:len() and x == name then
			return true
		end
	end
	return false
end

function enable_lod_dependants(addon)
	local addon_name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(addon)

	for i=1, GetNumAddOns() do
		local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(i)
		local isdep = find_iterate_over(addon_name, GetAddOnDependencies(name))
		local ondemand = IsAddOnLoadOnDemand(name)
		if isdep and not enabled and ondemand then
			-- ACP_EnableRecurse(name)
			EnableAddOn(name)
		end
	end
end

function recursive_iterate_over(...)
	for i=1,select("#", ...) do
		local x = select(i, ...)
		if x and x:len() then
			ACP_EnableRecurse(x)
		end
	end
end

